/*
   *
   * bcount.c - Bytes Counter
   *
   * This program makes statistics transferred and received
   * data over the network interfaces (TX / RX).
   *
   * Copyright (c) 2002 Aleksander Augustowski
   *
   * Depend:          Curses (aka Ncurses)
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/

#include "common.h"
#include "bcount.h"
#include "bserver.h"



int main(int argc, char **argv)
{

 int c, wlog = 0, rlog = 0, sum = 0, help = 0;
 char key;
 fd_set readfd;
 struct timeval timeout;


 exec_name = strrchr (argv[0], PATH_SEPARATOR);
 if (!exec_name)
    exec_name = argv[0];
    else
        ++exec_name;


    get_date();

    while ((c = getopt_long(argc, argv, "i:f:e:wrsvVh", longopts, 0)) != EOF)
       switch (c)
           {
              case 'i':
                 device = optarg;
                 break;
              case 'f':
                 log_file = optarg;
                 break;
              case 'e':
                 html_file = optarg;
                 html_answer++;
                 break;
              case 'w':
                 wlog++;
                 break;
              case 'r':
                 rlog++;
                 break;
              case 's':
                 sum++;
                 break;
              case 'v':
              case 'V':
                 print_version();
                 puts("");
                 exit(0);
              case 'h':
                 help++;
                 break;
              default:
                 print_info();
           }

 /* Maintenance of signals */
 signal(SIGSEGV, sigsegv);
 signal(SIGTSTP, sigvoid);
 signal(SIGTERM, sigterm);
 signal(SIGQUIT, sigterm);
 signal(SIGINT, sigterm);


 search_device (device, CHECK_DEVICE);

 if (wlog != 0)
    {
      check_if_old_log();
      write_log();

      printf ("%s: Saved RX:", device);
      search_device (device, RX_BYTES);
      printf ("%d B and ", int_digit_info);
      search_device (device, TX_BYTES);
      printf ("TX:%d B in log file.\n", int_digit_info);
      exit(0);
    }



 if (rlog != 0)
   {
     check_if_old_log();
     check_log_read();
     if (check_read == 1)
        read_error();

     read_log();
     exit(0);
    }


 if (sum != 0)
   {
     check_if_old_log();
     check_log_read();
     if (check_read == 1)
        read_error();
     summary_bytes();
     exit(0);
    }


 if (help != 0)
    {
      print_usage();
      exit(0);
    }


 if (html_answer != 0)
    check_html_file();


 check_if_old_log();
 get_file_info(log_file);

 init_screen();
 init_screen_color();
 draw_screen();
 make_search_window();
 total_bytes();
 setsyx (25, 80);

 /* Thread make_daemon() function */
 pthread_create (&thread_id, NULL, &make_daemon, NULL);
 pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

      do
        {
           FD_ZERO (&readfd);
           FD_SET (0, &readfd);
           timeout.tv_sec = 0;
           timeout.tv_usec = 1;

           switch ((select (1, &readfd, NULL, NULL, &timeout)) > 0)
              {
                 case 0:
                      if ((FD_ISSET (0, &readfd)) == 0)
                         {
                            move(2, 36);
                            rotate_dash();
                            refresh_now_window();
                         }
                      break;
                 case -1:
                      if (errno != EINTR)
                         {
                            perror (PROGRAM_NAME": main() select");
                            exit(-1);
                            break;
                         }
                 case 1:
                 case 2:
                      if (FD_ISSET (0, &readfd))
                         {
                             remove_dash();
                             switch (key = getch())
                                  {
                                      case 'm':
                                      case 'M':
                                           make_mail();
                                           break;
                                      case 's':
                                      case 'S':
                                           searching();
                                           break;
                                      case 'q':
                                      case 'Q':
                                           close_screen();
                                           break;
                                      default:
                                           break;

                                  }

                         }
                      break;

              }

        }
        while (tolower(key) != 'q');

        print_exit();

  return(0);

}


/* Initialize Windows ;-) */
int init_screen ()
{
   
   int y_max, x_max;

   if ((main_window = initscr()) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): main_window");
        return -1;
      }
   getmaxyx (stdscr, y_max, x_max);

   if ((void_window = derwin (stdscr, y_max, x_max, 0, 0)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): void_window");
        return -1;
      }

   if ((interface_window = derwin (stdscr, 1, x_max - 2, 0, 1)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): interface_window");
        return -1;
      }

   if ((now_window = derwin (stdscr, y_max - 15, x_max - 45, 2, 2)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): now_window");
        return -1;
      }
   getmaxyx (now_window, now_y_max, now_x_max);


   if ((total_window = derwin (stdscr, y_max - 18, x_max - 45, 12, 2)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): total_window");
        return -1;
      }

   if ((search_window = derwin (stdscr, y_max - 15, x_max - 40, 2, 38)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): search_window");
        return -1;
      }

   if ((mail_window = derwin (stdscr, y_max - 15, x_max - 40, 2, 38)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): mail_window");
        return -1;
      }

   if ((status_window = derwin (stdscr, 0, x_max - 10, y_max - 2, 1)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): status_window");
        return -1;
      }

   if ((mail_status_window = derwin (stdscr, 0, x_max - 72, y_max - 2, 71)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): status_window");
        return -1;
      }

   if ((server_window = derwin (stdscr, y_max - 15, x_max - 40, 12, 38)) == NULL)
      {
        perror (PROGRAM_NAME": init_screen(): server_window");
        return -1;
      }
   getmaxyx (server_window, serv_y_max, serv_x_max);

   cbreak();
   noecho();

   return(0);
}


int init_screen_color()
{

    if (!has_colors())
       {
         printw ("This Terminal does not support color.\n\
If support, please make sure than TERM= environment set properly\n");
         refresh();
         sleep(10);
       }

    start_color();

    init_pair (1, COLOR_WHITE, COLOR_BLUE);
    init_pair (2, COLOR_BLACK, COLOR_WHITE);
    init_pair (3, COLOR_BLACK, COLOR_BLUE);
    init_pair (4, COLOR_RED, COLOR_WHITE);
    init_pair (5, COLOR_GREEN, COLOR_WHITE);
    init_pair (6, COLOR_GREEN, COLOR_BLUE);
    init_pair (7, COLOR_RED, COLOR_BLUE);
    init_pair (7, COLOR_WHITE, COLOR_RED);

 return(0);
}


/* Window border, fill, text, etc. ... */
int draw_screen()
{
    char *interface_encap[] = { "Loopback", "Ethernet", "Point-To-Point" };
    char *interface_item = "  Interface:          IP:                  Encap:                 Status:";


    wattron (void_window, COLOR_PAIR(1));
    wattron (void_window, A_NORMAL);
    box (void_window, ACS_VLINE, ACS_HLINE);
    fill_window (void_window);
    mvwaddstr (void_window, 20, 4, "Keys: 's' - Search  'm' - Mail");
    mvwaddstr (void_window, 21, 10, "'q' - Quit");
    mvwaddstr (void_window, 19, 4, "BYTES COUNTER ver.");
    wattron (void_window, A_UNDERLINE);
    mvwprintw (void_window, 19, 23, "%s", VERSION);
    wattron (void_window, A_NORMAL);
    wrefresh (void_window);

    wattron (interface_window, COLOR_PAIR(2));
    fill_panel (interface_window);
    mvwprintw (interface_window, 0, 0, "%s", interface_item);

    wattron (interface_window, COLOR_PAIR(4));

    if ((strcmp (device, "lo")) == 0)
        mvwprintw (interface_window, 0, 51, interface_encap[0]);
        else
            mvwaddstr (interface_window, 0, 51, "Unknown");

    if ((strcmp (device, "eth0")) == 0)
        mvwprintw (interface_window, 0, 51, interface_encap[1]);
    if ((strcmp (device, "eth1")) == 0)
        mvwprintw (interface_window, 0, 51, interface_encap[1]);
    if ((strcmp (device, "eth2")) == 0)
        mvwprintw (interface_window, 0, 51, interface_encap[1]);

    if (((strcmp (device, "ppp0")) == 0) || ((strcmp (device, "ippp0")) == 0))
        mvwprintw (interface_window, 0, 51, interface_encap[2]);
    if (((strcmp (device, "ppp1")) == 0) || ((strcmp (device, "ippp1")) == 0))
        mvwprintw (interface_window, 0, 51, interface_encap[2]);
    if (((strcmp (device, "ppp2")) == 0) || ((strcmp (device, "ippp2")) == 0))
        mvwprintw (interface_window, 0, 51, interface_encap[2]);


    mvwprintw (interface_window, 0, 14, "%s", device);
    read_iface_ip();

    wattron (interface_window, COLOR_PAIR(5));
    mvwaddstr (interface_window, 0, 75, "UP");
    wrefresh (interface_window);

    wattron (now_window, COLOR_PAIR(1));
    box (now_window, ACS_VLINE, ACS_HLINE);
    text_window_in_middle (now_window, " Now RX/TX: ", "top");
    fill_window (now_window);
    mvwaddstr (now_window, 2, 3, "Transmited Data:");
    mvwaddstr (now_window, 3, 11, "Packets:");
    mvwaddstr (now_window, 4, 12, "Errors:");
    mvwaddstr (now_window, 5, 5, "Received Data:");
    mvwaddstr (now_window, 6, 11, "Packets:");
    mvwaddstr (now_window, 7, 12, "Errors:");
    wrefresh (now_window);

    wattron (total_window, COLOR_PAIR(1));
    wattron (total_window, A_NORMAL);
    box (total_window, ACS_VLINE, ACS_HLINE);
    text_window_in_middle (total_window, " Total RX/TX: (in log file) ", "top");
    fill_window (total_window);
    search_device (device, TX_BYTES);
    mvwprintw (total_window, 2, 3, "Transmited Data: %.1f KB", float_digit_Kbytes);
    search_device (device, RX_BYTES);
    mvwprintw (total_window, 3, 5, "Received Data: %.1f KB", float_digit_Kbytes);
    wrefresh (total_window);

    wattron (server_window, COLOR_PAIR(1));
    box (server_window, ACS_VLINE, ACS_HLINE);
    fill_window (server_window);
    text_window_in_middle (server_window, " Status of connections [Server] ", "top");
    wrefresh (server_window);

    wattron (status_window, COLOR_PAIR(2));
    fill_panel (status_window);
    mvwprintw (status_window, 0, 0, "  Log file: %s (%d Bytes | UID: %d GID: %d) ", log_file, file_size, file_uid, file_gid);
    wrefresh (status_window);

    wattron (mail_status_window, COLOR_PAIR(2));
    fill_panel (mail_status_window);
    mvwaddch (mail_status_window, 0, 0, ACS_VLINE);
    wrefresh (mail_status_window);

    refresh();
    curs_set(INVISIBLE);

  return(0);
}


int make_search_window()
{

    wattron (search_window, COLOR_PAIR(1));
    box (search_window, ACS_VLINE, ACS_HLINE);
    fill_window (search_window);
    text_window_in_middle (search_window, " Search RX/TX: (in log file) ", "top");
    wattron (search_window, COLOR_PAIR(2));
    text_window_in_middle (search_window, "   Press 's' for search in log file  ", "bottom");
    mvwaddstr (search_window, 2, 20, "          ");
    mvwaddstr (search_window, 4, 20, "          ");


    wattron (search_window, COLOR_PAIR(1));
    mvwaddstr (search_window, 6, 8, "Transmited Data:");
    mvwaddstr (search_window, 7, 10, "Received Data:");
    mvwaddstr (search_window, 2, 9, "Date from:");
    mvwaddstr (search_window, 4, 11, "Date to:");
    wattron (search_window, COLOR_PAIR(3));
    mvwprintw (search_window, 3, 11, "Example: %s", current_date);
    wattron (search_window, COLOR_PAIR(1));
    wrefresh (search_window);
    return(0);
}


int make_mail_window()
{

    wattron (mail_window, COLOR_PAIR(1));
    box (mail_window, ACS_VLINE, ACS_HLINE);
    text_window_in_middle (mail_window, " Check and mail ", "top");
    fill_window (mail_window);
    mvwaddstr (mail_window, 2, 2, "If received");
    wattron (mail_window, COLOR_PAIR(2));
    mvwaddstr (mail_window, 2, 14, "      ");
    wattron (mail_window, COLOR_PAIR(1));
    mvwaddstr (mail_window, 2, 21, "MB then send mail");
    mvwaddstr (mail_window, 4, 2, "to");
    wattron (mail_window, COLOR_PAIR(2));
    mvwaddstr (mail_window, 4, 5, "                                 ");
    wrefresh (mail_window);
    return(0);
}


/* Close initialized Windows */
int close_screen()
{

   delwin (void_window);
   delwin (interface_window);
   delwin (now_window);
   delwin (search_window);
   delwin (status_window);
   delwin (mail_window);
   delwin (server_window);
   delwin (main_window);
   erase();
   refresh();
   endwin();

  return(0);
}


/* Fill Window (char 32 -> space) */
int fill_window (WINDOW *win)
{

 short int i, z, max_x, max_y;
 short int x = 1, y = 1;

 getmaxyx (win, max_x, max_y);
 for (z = y; z < max_y - 1; z++)
     {
        move (x, y);
        for (i = x; i < max_x - 1; i++)
                mvwprintw (win, i, y, " ");

        y++;
     }
 return(0);

}


/* Fill Window (char 32 -> space) */
int fill_panel (WINDOW *win)
{

 short int x, y_max, x_max;

 getmaxyx (win, y_max, x_max);
 for (x = 0; x < x_max; x++)
       mvwprintw (win, 0, x, " ");

 return(0);

}


/* Get string lenght and put in the middle of Window */
int text_window_in_middle (WINDOW *win, char *label_window, char *place)
{
   int x = 0, y = 0, x_max, y_max;
   int c, e;

   getmaxyx (win, y_max, x_max);
   if (place == "bottom")
      y = y_max - 1;

   if (place == "top")
      y = 0;

   c = strlen (label_window);
   e = ((x_max / 2) + x ) - (c / 2);
   mvwprintw (win, y, e, "%s", label_window);

 return(0);
}


/* I was take this from UConio. Sorry Pablo Picasso :-) */
void clrscr(void)
{
   fprintf(stdout, "\033[2J");
   fprintf(stdout, "\033[%i;%iH", 0, 0);
   fprintf(stdout, "\033[0;0;39m");
}


/* Print system date */
int get_date()
{

 time_t bintime;
 struct tm *current_time;

 time(&bintime);
 current_time = localtime(&bintime);

 snprintf (current_date, sizeof(current_date), "%d-%02d-%02d", current_time->tm_year + 1900, current_time->tm_mon + 1, current_time->tm_mday);

 return(0);
}


/* Get file info like: size, uid and gid */
int get_file_info (char *filename)
{
 struct stat file_info;

 if (stat (filename, &file_info) == -1)
    {
       perror (PROGRAM_NAME": get_file_size()");
       exit(1);
    }

 file_size = file_info.st_size;
 file_uid = file_info.st_uid;
 file_gid = file_info.st_gid;

 return(0);

}


/* Search records of device */
int search_device (char *dev, int search_for)
{

 FILE *fp;
 char buffer[1024];
 char temp_info[20];
 size_t bytes_read;
 char device_buffer[200];
 char device_test[20];
 char* temp_buffer;
 int device_buffer_lenght = 0, records = 0;
 int flag = 666, n = 0, i = 0, c = 0;

 if ((fp = fopen(DEV_FILE, "r")) == NULL)
    {
        printw ("File %s not found.\n\
This program don't execute without \"proc file system\", exiting...\n", DEV_FILE);
        perror ("search_device()");
    }

  fseek (fp, 199L, SEEK_SET); /* 199 Bytes have trash texts in log_file */
  bytes_read = fread (buffer, 1, sizeof(buffer), fp);
  fclose(fp);

  if (bytes_read == 0 || bytes_read == sizeof(buffer))
    {
      perror ("search_device(): Buffer underrun");
      exit(1);
    }

  buffer[bytes_read] = '\0';
  temp_buffer = strstr (buffer, dev);
  if (temp_buffer == NULL)
     device_error();
  
  while (temp_buffer[n] != ':') 
    n++;
    
  strncpy(device_test, temp_buffer, n);
    
  device_test[n] = '\0';
  
  if (strcmp(device_test, dev))
     device_error();

  while (search_for == flag)    /* Decide: exit or not */
     return(0);

  while (temp_buffer[n] != '\n') 
    n++;

  strncpy(device_buffer, temp_buffer, n);

  n = 0;
  

  while (device_buffer[n] != ':')
     n++;

  device_buffer_lenght = strlen(device_buffer);
  while (n != device_buffer_lenght)
     {
        if ((isspace(device_buffer[n])) && (isdigit(device_buffer[n+1])) == 0)
             ;
             else
                 {
                    if (((isspace(device_buffer[n])) && (isdigit(device_buffer[n+1])) != 0) || ((device_buffer[n] == ':') && (isdigit(device_buffer[n+1]) != 0)))
                      {
                          records++;
                          if (records == search_for)
                             {
                                i = n+1;
                                while ((isspace(device_buffer[i])) == 0)
                                    {
                                        temp_info[c] = device_buffer[i];
                                        c++;
                                        i++;
                                    }
                             }

                      }

                 }
        n++;
      }


  temp_info[c] = '\0';

   
  int_digit_info = atoi(temp_info);
   float_digit_bytes = atof(temp_info);
    float_digit_Kbytes = float_digit_bytes / 1024;
     float_digit_Mbytes = float_digit_Kbytes / 1024;
   
 return(0);
}


/* Refresh info in Now Window */
int refresh_now_window()
{
   float tx_for_html, rx_for_html;

   wattron (now_window, A_UNDERLINE);
   search_device (device, TX_BYTES);

   if (float_digit_Kbytes >= 1024)
      {
         mvwaddstr (now_window, 2, 20, "              ");
         mvwprintw (now_window, 2, 20, "%.1f MB", float_digit_Mbytes);
         if (html_answer != 0)
            tx_for_html = float_digit_Mbytes;
      }
      else
             mvwprintw (now_window, 2, 20, "%.1f KB", float_digit_Kbytes);

   search_device (device, TX_PACKETS);
   mvwprintw (now_window, 3, 20, "%d", int_digit_info);
   search_device (device, TX_ERRS);
   mvwprintw (now_window, 4, 20, "%d", int_digit_info);
   search_device (device, RX_BYTES);
   if (float_digit_Kbytes >= 1024)
      {
         mvwaddstr (now_window, 5, 20, "              ");
         mvwprintw (now_window, 5, 20, "%.1f MB", float_digit_Mbytes);
         if (no_more_mail == 0)
           {
             if (float_digit_Mbytes > atoi(get_mail_digit))
               {
                  snprintf (data_to_send_rx, sizeof(data_to_send_rx), "Hi ;-)\nYour %s interface get: %.1f MB\n\nRegards\n", device, float_digit_Mbytes);

                  if (strlen(get_mail_adress) > 0 && strlen(get_mail_digit) > 0)
                     {
                         sendmail ("bcount", get_mail_adress, "From Bytes Counter", data_to_send_rx);

                         wattron (mail_status_window, COLOR_PAIR(2));
                         wattroff (mail_status_window, A_BLINK);
                         mvwaddstr (mail_status_window, 0, 1, "       ");
                         mvwaddstr (mail_status_window, 0, 1, "Sent");
                         wrefresh (mail_status_window);
                         no_more_mail = 1;
                     }
               }

           }
         if (html_answer != 0)
            rx_for_html = float_digit_Mbytes;

      }
      else
          mvwprintw (now_window, 5, 20, "%.1f KB", float_digit_Kbytes);

   search_device (device, RX_PACKETS);
   mvwprintw (now_window, 6, 20, "%d", int_digit_info);
   search_device (device, RX_ERRS);
   mvwprintw (now_window, 7, 20, "%d", int_digit_info);
   wattron (now_window, A_NORMAL);
   wrefresh (now_window);
   refresh();
   if (html_answer != 0)
       generate_html(html_file, tx_for_html, rx_for_html);

   return(0);
}


int read_iface_ip()
{
   struct in_addr ipaddr;
   struct sockaddr_ll saddr;

   ip_socket = socket (PF_INET, SOCK_STREAM, 0);
   if (ip_socket < 0)
       {
          print_exit();
          perror (PROGRAM_NAME": socket()");
          exit(1);
       }

 get_iface_ip (device, &saddr, &ipaddr);

 mvwprintw (interface_window, 0, 27, "%s", inet_ntoa(ipaddr));
 close(ip_socket);
 wrefresh(interface_window);

 return(0);
}


int get_iface_ip (char *ifname, struct sockaddr_ll *hwaddr, struct in_addr *ip)
{

 struct ifreq ifr;

 sprintf (ifr.ifr_name, "%s", ifname);
 if (ioctl (ip_socket, SIOCGIFHWADDR, &ifr))
    {
        print_exit();
        perror (PROGRAM_NAME": ioctl()");
        exit(1);
    }
    
 memcpy (hwaddr->sll_addr, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
 ioctl (ip_socket, SIOCGIFINDEX, &ifr);
 hwaddr->sll_ifindex = ifr.ifr_ifindex;
 ioctl (ip_socket, SIOCGIFADDR, &ifr);
 memcpy ((char *) ip, ifr.ifr_addr.sa_data + 2, 4);

 return(0);
}


/* Read to table date and search log_file */
int searching()
{
 FILE *fp;
 unsigned int int_rx_b = 0, int_tx_b = 0;
 float float_rx_b, float_rx_kb, float_rx_mb;
 float float_tx_b, float_tx_kb, float_tx_mb;
 int k = 0, dates_records = 0;
 char date_from[10];
 char date_to[10];

 wattron (search_window, COLOR_PAIR(1));
 wattroff (search_window, A_UNDERLINE);
 mvwaddstr (search_window, 6, 25, "             ");
 mvwaddstr (search_window, 7, 25, "             ");
 wattron (search_window, COLOR_PAIR(2));
 mvwaddstr (search_window, 2, 20, "          ");
 mvwaddstr (search_window, 4, 20, "          ");
 wmove (search_window, 2, 20);
 wrefresh (search_window);

 noecho();
 curs_set(VISIBLE);

 top:
 for (k = 0; k < 10; k++)
     {
         head:
         if ((date_from[k] = getch()) == 9)
            break;

         if (date_from[k] == 'q' || date_from[k] == 'Q')
            print_exit();

         if (date_from[k] == KEY_BKSP)
           {
              if (k == 0)
                 goto top;
              mvwaddstr (search_window, 2, 20 + (k - 1), " ");
              wmove (search_window, 2, 20 + (k - 1));
              wrefresh (search_window);
              k--;
              goto head;
           }

         if (date_from[k] == KEY_ESC)
            {
               mvwaddstr (search_window, 2, 20, "          ");
               wmove (search_window, 2, 20);
               wrefresh (search_window);
               goto end;
            }

         if (k == 4 && date_from[k] != '-')
            goto head;

         if (k == 7 && date_from[k] != '-')
            goto head;

         if (date_from[k] == '\n')
             k--;
               else
                  waddch (search_window, date_from[k]);
                  wrefresh (search_window);

     }

 date_from[k] = '\0';

 wmove (search_window, 4, 20);
 wrefresh (search_window);

 top2:
 for (k = 0; k < 10; k++)
     {
         head2:
         if ((date_to[k] = getch()) == 9)
            break;

         if (date_to[k] == 'q' || date_to[k] == 'Q')
            print_exit();

         if (date_to[k] == KEY_BKSP)
           {
              if (k == 0)
                 goto top2;
              mvwaddstr (search_window, 4, 20 + (k - 1), " ");
              wmove (search_window, 4, 20 + (k - 1));
              wrefresh (search_window);
              k--;
              goto head2;
           }

         if (date_to[k] == KEY_ESC)
            {
               mvwaddstr (search_window, 4, 20, "          ");
               wmove (search_window, 4, 20);
               wrefresh (search_window);
               goto end;
            }

         if (k == 4 && date_to[k] != '-')
            goto head2;

         if (k == 7 && date_to[k] != '-')
            goto head2;

         if (date_to[k] == '\n')
             k--;
               else
                  waddch (search_window, date_to[k]);
                  wrefresh (search_window);

     }

 date_to[k] = '\0';

 check_log_read();
 fp = fopen(log_file, "r");

 while (fread (&log, sizeof(log), 1, fp) == 1)
    {
       while (strcmp (log.data, date_from) == 0 && strcmp (log.interface, device) == 0)
          {
             dates_records = log.record;
             goto found;
          }
    }

 found:
 fseek (fp, -44L, SEEK_CUR); /* 44 Bytes have struct log (one record) */

 while (fread (&log, sizeof(log), 1, fp) == 1)
    {
       if (strcmp (log.data, date_to) != 0 && strcmp (log.interface, device) == 0 && (dates_records == log.record))
         {
             int_rx_b += log.rx;
             int_tx_b += log.tx;
             dates_records++;
         }

       if (strcmp (log.data, date_to) == 0 && strcmp (log.interface, device) == 0)
         {
             int_rx_b += log.rx;
             int_tx_b += log.tx;
         }

    }

 fclose(fp);

 float_rx_b = int_rx_b;
 float_tx_b = int_tx_b;

 float_rx_kb = float_rx_b / 1024;
 float_tx_kb = float_tx_b / 1024;

 wattron (search_window, A_UNDERLINE);
 wattron (search_window, COLOR_PAIR(1));
 if (float_tx_kb >= 1024)
    {
        mvwaddstr (search_window, 6, 25, "              ");
        mvwprintw (search_window, 6, 25, "%.1f MB", float_tx_mb = float_tx_kb / 1024);
    }
     else
          mvwprintw (search_window, 6, 25, "%.1f KB", float_tx_kb);

 if (float_rx_kb >= 1024)
    {
        mvwaddstr (search_window, 7, 25, "              ");
        mvwprintw (search_window, 7, 25, "%.1f MB", float_rx_mb = float_rx_kb / 1024);
    }
     else
          mvwprintw (search_window, 7, 25, "%.1f KB", float_rx_kb);

 mvwaddstr (search_window, 3, 14, "");
 wattron (search_window, A_NORMAL);

 wrefresh (search_window);

 end:
 curs_set(INVISIBLE);
 return(0);
}


/* Sum of date from log_file */
int total_bytes()
{

 FILE *fp;
 unsigned int int_rx_suma_b = 0, int_tx_suma_b = 0, rec = 0;
 float float_rx_suma_b, float_tx_suma_b;
 float float_rx_suma_kb, float_tx_suma_kb;
 float float_rx_suma_mb, float_tx_suma_mb;

 check_log_read();
 fp = fopen(log_file, "r");

 while (fread (&log, sizeof(log), 1, fp) == 1)
  if (strcmp (log.interface, device) == 0)
    {
      int_rx_suma_b += log.rx;
      int_tx_suma_b += log.tx;
      rec++;
    }

 fclose(fp);

 float_tx_suma_b = int_tx_suma_b;
 float_rx_suma_b = int_rx_suma_b;
 float_tx_suma_kb = float_tx_suma_b / 1024;
 float_rx_suma_kb = float_rx_suma_b / 1024;

 mvwaddstr (total_window, 2, 20, "             ");
 mvwaddstr (total_window, 3, 20, "             ");
 mvwaddstr (total_window, 2, 3, "Transmited Data:");
 mvwaddstr (total_window, 3, 5, "Received Data:");
 mvwaddstr (total_window, 4, 11, "Records:");
 wattron (total_window, A_UNDERLINE);
 if (float_tx_suma_kb >= 1024)
    {
        mvwaddstr (total_window, 2, 20, "              ");
        mvwprintw (total_window, 2, 20, "%.1f MB", float_tx_suma_mb = float_tx_suma_kb / 1024);
    }
     else
          mvwprintw (total_window, 2, 20, "%.1f KB", float_tx_suma_kb);

 if (float_rx_suma_kb >= 1024)
    {
        mvwaddstr (total_window, 3, 20, "              ");
        mvwprintw (total_window, 3, 20, "%.1f MB", float_rx_suma_mb = float_rx_suma_kb / 1024);
    }
     else
          mvwprintw (total_window, 3, 20, "%.1f KB", float_rx_suma_kb);

 mvwprintw (total_window, 4, 20, "%d", rec);

 wattron (total_window, A_NORMAL);
 wrefresh (total_window);

 return(0);
}


/* Summary bytes for interface from log_file */
int summary_bytes()
{

 FILE *fp;
 unsigned int int_rx_suma_b = 0, int_tx_suma_b = 0;
 float float_rx_suma_b, float_tx_suma_b;
 float float_rx_suma_kb, float_tx_suma_kb;

 check_log_read();
 fp = fopen(log_file, "r");
    
 while (fread (&log, sizeof(log), 1, fp) == 1)
  if (strcmp (log.interface, device) == 0)
   {
      int_rx_suma_b += log.rx;
      int_tx_suma_b += log.tx;
   }
        
 fclose(fp);


 float_tx_suma_b = int_tx_suma_b;
 float_rx_suma_b = int_rx_suma_b;

 printf ("Summary Bytes for: %s\n", device);
 printf ("\nTX: %.1f KB\n", float_tx_suma_kb = float_tx_suma_b / 1024);
 printf ("RX: %.1f KB\n\n", float_rx_suma_kb = float_rx_suma_b / 1024);

 return(0);
}


/* Read data from log_file */
int read_log()
{

 FILE *fp;

 check_log_read();
 fp = fopen(log_file, "r");

 printf ("Reading Records for: %s\n\n", device);

 while (fread (&log, sizeof(log), 1, fp) == 1)
  if (strcmp (log.interface, device) == 0)
    {
       printf ("%s\n", log.data);
       printf ("TX: %d B\n", log.tx);
       printf ("RX: %d B\n", log.rx);
       printf ("Record: %d\n\n", log.record);
    }

 fclose(fp);

 return(0);
}


/* Check if we must delete old log file :-( */
int check_if_old_log()
{

 FILE *fp3;
 long curpos;
 int rec = 0, check = 0;
 
 if ((fp3 = fopen(log_file, "r")) == NULL)
    {
      check_log_append();
      return(0);
    }

 get_file_info(log_file);
 if (file_size == 0)
    return(0);

 while (fread (&log, sizeof(log), 1, fp3) == 1)
      rec++;

/* curpos = ftell(fp3);
 printf("curpos = %ld\n", curpos);
 printf("rec = %d\n", rec);
 printf("%d\n", curpos / rec); */

 if (file_size == 40 || (check = ((curpos = ftell(fp3)) / rec)) != 44)
     {
        printf ("Sorry but you have old \033[1;37m%s\033[0m. You must delete this old one.\n\
Version 0.4 and following versions using records in log structure.\n\
Please be sure that you don't write in log file with bcount older than 0.4,\n\
because result in \033[2;37mSearching Window\033[0m will be other than we expecting!\n\
If you want good result with statistics of bytes please delete this file\n\
and use only bcount 0.4 and following for writting in \033[1;37m%s\033[0m.\n\
I promise never more changeing this trouble spot file ;-)\n", log_file, log_file);
        fclose(fp3);
        exit(1);
     }

 fclose(fp3);
 return(0);
}


/* Gets number of records from log_file */
int get_nr_record()
{

 FILE *fp;

 check_log_read();
 record_number = 0;
 fp = fopen(log_file, "r");

 while (fread (&log, sizeof(log), 1, fp) == 1)
    record_number++;

 fclose(fp);

 return(0);
}


/* Write data to log_file */
int write_log()
{

 FILE *fp;

 check_log_append();
 get_nr_record();
 fp = fopen(log_file, "a");

   strcpy(log.data, current_date);
   strcpy(log.interface, device);
   search_device (device, RX_BYTES);
   log.rx = int_digit_info;
   search_device (device, TX_BYTES);
   log.tx = int_digit_info;
   log.record = record_number + 1;

   fwrite (&log, sizeof(log), 1, fp);
   fclose (fp);

 return(0);
}


/* Checking if we can make html_file file */
int check_html_file()
{

 if ((hf = fopen(html_file, "w")) == NULL)
      {
         perror("Error, cannot open html file");
         exit(1);
      }

 fclose(hf);
 return(0);
}


/* Checking if log_file exist */
int check_log_read()
{

 FILE *fp;

 if ((fp = fopen(log_file, "r")) == NULL)
    return check_read = 1;

 fclose(fp);
 return(0);
}


/* Checking if we can write to log_file */
int check_log_append()
{
 FILE *fp2;
 
 if ((fp2 = fopen(log_file, "a")) == NULL)
     write_error();

 return(0);
}


/* I got this from mc :-) */
void rotate_dash (void)
{
    static char rotating_dash [] = "|/-\\";
    static int position = 0;

    if (position >= sizeof (rotating_dash)-1)
       position = 0;
    wattroff (now_window, A_UNDERLINE);
    mvwaddch (now_window, 0, now_x_max - 2, rotating_dash[position]);
    wrefresh (now_window);
    position++;
}


void remove_dash (void)
{
    wattroff (now_window, A_UNDERLINE);
    mvwaddch (now_window, 0, now_x_max - 2, ACS_HLINE);
    wrefresh (now_window);
}


/* I take this from sntop ...and little changing */
void generate_html(char *file, float tx, float rx)
{
        extern char *tzname[2];
        time_t t;
        char *s;
        int refresh = 2;

        if ((hf = fopen(file, "w")) == NULL)
           perror("Error, cannot open html file");

        fputs("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n", hf);
        fputs("<HTML>\n<HEAD>\n", hf);
        fprintf(hf, "<META http-equiv=\"Refresh\" content=\"%d\">\n", refresh);
        fputs("<TITLE>Bytes Counter HTML output</TITLE>\n</HEAD>\n", hf);
        fputs("<BODY TEXT=\"#000000\" BGCOLOR=\"#FFFFFF\">\n\n", hf);
        fputs("<P><B>Bytes Counter status - automatically generated html</B></P><BR>\n\n", hf);
        fputs("<TABLE BORDER CELLPADDING=5>\n<TR VALIGN=top>\n", hf);
        fputs("<TD BGCOLOR=#EEEEEE><B>Interface</B></TD>\n<TD BGCOLOR=#EEEEEE>", hf);
        fputs("<B>Received RX:</B></TD>\n<TD BGCOLOR=#EEEEEE><B>Transmited TX:</B></TD>\n", hf);
        fputs("</TR>\n", hf);

        fprintf(hf, "<TR>\n<TD BGCOLOR=#EEEEEE>%s</TD>\n", device);
        fprintf(hf, "<TD BGCOLOR=#EEEEEE>%.1f MB</TD>\n", rx);
        fprintf(hf, "<TD BGCOLOR=#EEEEEE>%.1f MB</TD>\n", tx);

        fputs("</TABLE>\n\n", hf);

        if (time(&t) != (time_t) -1)
              if ((s = ctime(&t)) && tzname[0])
                       fprintf(hf, "<P></P><BR>\n<P><SMALL>Last updated: %s %s</SMALL></P>\n\n", s, tzname[0]);

        fputs("<P><SMALL>Generated by <A HREF=\"http://www.ins.com.pl/~alex/bcount/bcount.html\">", hf);
        fputs("Bytes Counter</A> - " VERSION "</SMALL></P>\n\n", hf);
        fputs("</BODY>\n</HTML>", hf);
        fclose(hf);

}


/* If we received couple of bytes then we got mail */
int make_mail()
{

 int k;

 wattron (mail_status_window, A_NORMAL | COLOR_PAIR(2));
 mvwaddstr (mail_status_window, 0, 1, "       ");
 make_mail_window();
 wrefresh(mail_status_window);
 move (4, 52);
 wmove (mail_window, 2, 14);
 noecho();
 curs_set(VISIBLE);

 mail_top:
 for (k = 0; k < 6; k++)
     {
         mail_head:

         if ((get_mail_digit[k] = getch()) == 5)
            break;

         if (get_mail_digit[k] == 'q' || get_mail_digit[k] == 'Q')
            print_exit();

         if (get_mail_digit[k] == KEY_BKSP)
           {
              if (k == 0)
                 goto mail_top;
              mvwaddstr (mail_window, 2, 14 + (k - 1), " ");
              wmove (mail_window, 2, 14 + (k - 1));
              wrefresh (mail_window);
              k--;
              goto mail_head;
           }

         if (get_mail_digit[k] == KEY_ESC)
            {
               no_more_mail = 1;
               mvwaddstr (mail_window, 2, 14, "      ");
               wrefresh (mail_window);
               goto mail_end;
            }

         if (get_mail_digit[k] == '\n')
            break;

         if (isdigit(get_mail_digit[k]) == 0)
            goto mail_head;
            
         waddch (mail_window, get_mail_digit[k]);
         wrefresh (mail_window);

     }

   get_mail_digit[k] = '\0';

   move (6, 43);
   wmove (mail_window, 4, 5);

   mail_top_2:
   for (k = 0; k < 33; k++)
     {
         mail_head_2:

         if ((get_mail_adress[k] = getch()) == 32)
            break;

         if (get_mail_adress[k] == 'q' || get_mail_adress[k] == 'Q')
            print_exit();

         if (get_mail_adress[k] == KEY_BKSP)
           {
              if (k == 0)
                 goto mail_top_2;
              mvwaddstr (mail_window, 4, 5 + (k - 1), " ");
              wmove (mail_window, 4, 5 + (k - 1));
              wrefresh (mail_window);
              k--;
              goto mail_head_2;
           }

         if (get_mail_adress[k] == KEY_ESC)
            {
               no_more_mail = 1;
               mvwaddstr (mail_window, 4, 5, "                                 ");
               wrefresh (mail_window);
               goto mail_end;
            }

         if (get_mail_adress[k] == '\n')
            break;

         waddch (mail_window, get_mail_adress[k]);
         wrefresh (mail_window);

     }

   get_mail_adress[k] = '\0';

   no_more_mail = 0;

   if (strlen(get_mail_adress) > 0 && strlen(get_mail_digit) > 0)
      {
         wattron (mail_status_window, A_BLINK | COLOR_PAIR(4));
         mvwaddstr (mail_status_window, 0, 1, "Waiting");
         wrefresh (mail_status_window);
      }
      
  mail_end:
  curs_set(INVISIBLE);
  make_search_window();

  return(0);
}


/* Send mail if in Now Windows is ?????? MB of received bytes */
int sendmail (const char *from, const char *to, const char *subject, const char *text)
{
  char cmd[256];
  FILE *mail;

    sprintf (cmd, MTA, to);
    mail = popen(cmd, "w");
    if (mail == NULL)
       {
         perror (PROGRAM_NAME": sendmail() popen");
         return -1;
       }
    fprintf (mail, "From: %s\n", from);
    fprintf (mail, "To: %s\n", to);
    fprintf (mail, "Subject: %s\n", subject);
    fprintf (mail, "%s\n", text);
    pclose(mail);

  return(0);
}


/* Standalone Server for foreign connections */
void* make_daemon (void* unused)
{
 int alen, z, so_reuseaddr = TRUE;
 int y_serv = 0;
 char received_version[10];
 struct sockaddr_in server_addr;
 struct sigaction sigchld_action;

 /* Initialize buffer */
 char all_datas_to_send[500];

 /* Yes ...All the Children will be dying ;-) */
 memset (&sigchld_action, 0, sizeof(sigchld_action));
 sigchld_action.sa_handler = &clean_up_child_process;
 sigaction (SIGCHLD, &sigchld_action, NULL);

 server_socket = socket (PF_INET, SOCK_STREAM, 0);
 if (server_socket < 0)
    {
       fill_window (server_window);
       mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
       mvwprintw (server_window, 3, 2, "socket(): %s", strerror(errno));
       wrefresh(server_window);
       return(0);
    }

 /* SO_REUSEADDR help if we want run several times `bcount -d' then
    bind() function can't exit with "Address is alredy in use" :-) */
 z = setsockopt (server_socket, SOL_SOCKET, SO_REUSEADDR, &so_reuseaddr, sizeof(so_reuseaddr));
 if (z > 0)
    {
       fill_window (server_window);
       mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
       mvwprintw (server_window, 3, 2, "setsockopt(): %s", strerror(errno));
       wrefresh(server_window);
       return(0);
    }

 /* Internet address for family addresses */
 server_addr.sin_family = PF_INET;

 /* Host order (little-endian or big-endian) 
 to network order (always big-endian) for Port */
 server_addr.sin_port = htons (SPORT);

 /* Bind socket with adress of any interface */
 server_addr.sin_addr.s_addr = INADDR_ANY;


 if (bind (server_socket, (struct sockaddr *) &server_addr, sizeof (server_addr)))
    {
       fill_window (server_window);
       mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
       mvwprintw (server_window, 3, 2, "bind(): %s", strerror(errno));
       wrefresh(server_window);
       return(0);
    }
    else
        {
           y_serv = 2;
           mvwaddstr (server_window, y_serv++, 2, "Bytes Counter Server running ...");
           mvwprintw (server_window, y_serv++, 2, "Version %s [Linux/Intel x86]" , SVERSION);
           mvwprintw (server_window, serv_y_max - 1, 3, " Port: %d ", SPORT);
           mvwprintw (server_window, serv_y_max - 1, serv_x_max - 6, " %d ", how_many_connections);
           y_serv++;
           wrefresh(server_window);
        }


 if (listen (server_socket, 5))
    {
       fill_window (server_window);
       mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
       mvwprintw (server_window, 3, 2, "listen(): %s", strerror(errno));
       wrefresh(server_window);
       return(0);
    }

 openlog ("bcount", LOG_PID, LOG_DAEMON);
 syslog (LOG_ERR, "Bytes Counter Server is running... Listening on port:%d sv=%s", SPORT, SVERSION);


 /* Neverendig story ...and Limahl :-) */
 while(1)
    {
       struct sockaddr_in client_addr;
       pid_t child_pid;
       
       alen = sizeof(server_addr);
       client_socket = accept (server_socket, (struct sockaddr *) &client_addr, &alen);
       if (client_socket < 0)
          {
             /* If get signal then continue */
             if (errno == EINTR)
                continue;
             else
                 {
                    fill_window (server_window);
                    mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
                    mvwprintw (server_window, 3, 2, "accept(): %s", strerror(errno));
                    wrefresh(server_window);
                    closelog();
                    return(0);
                 }
          }

       /* Read version received from client */
       z = read (client_socket, received_version, sizeof(received_version));
       if (z < 0)
          {
             fill_window (server_window);
             mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
             mvwprintw (server_window, 3, 2, "read(): %s", strerror(errno));
             wrefresh(server_window);
             closelog();
             return(0);
          }

       /* On the end of received message we put NULL byte */
       received_version[z] = 0;

       if (shutdown (client_socket, SHUT_RD) == -1)
           {
              fill_window (server_window);
              mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
              mvwprintw (server_window, 3, 2, "shutdown(): %s", strerror(errno));
              wrefresh(server_window);
              closelog();
              return(0);
           }

       how_many_connections++;

 if (y_serv >= serv_y_max - 3)
      {
          fill_window (server_window);
          y_serv = 2;
      }

 move(25, 80);
 mvwprintw (server_window, y_serv++, 2, "> Connection from %s:%i", inet_ntoa (client_addr.sin_addr), ntohs (client_addr.sin_port));
 mvwprintw (server_window, y_serv++, 5, "Client version %s", received_version);
 mvwprintw (server_window, serv_y_max - 1, serv_x_max - 6, " %d ", how_many_connections);
 wrefresh(server_window);

       syslog (LOG_ERR, "Connection from %s:%i cv=%s", inet_ntoa (client_addr.sin_addr), ntohs (client_addr.sin_port), received_version);

       /* Touching data for send to the client */
       search_device (device, RX_BYTES);
       snprintf (data_to_send_rx, sizeof(data_to_send_rx), "%d", int_digit_info);
       search_device (device, TX_BYTES);
       snprintf (data_to_send_tx, sizeof(data_to_send_tx), "%d", int_digit_info);
       snprintf (all_datas_to_send, sizeof(all_datas_to_send), "%s\n%s\n%s\n%s\n", device, data_to_send_rx, data_to_send_tx, SVERSION);

       child_pid = fork();
       if (child_pid == 0)
         {
            close (STDIN_FILENO);
            close (STDOUT_FILENO);
            close (server_socket);

            z = write (client_socket, all_datas_to_send, sizeof(all_datas_to_send));
            if (z < 0)
               {
                  mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
                  mvwprintw (server_window, 3, 2, "write(): %s", strerror(errno));
                  wrefresh(server_window);
                  closelog();
                  return(0);
               }
               
            /* close (alen); */
            if (shutdown (client_socket, SHUT_WR) == -1)
                {
                    mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
                    mvwprintw (server_window, 3, 2, "shutdown(): %s", strerror(errno));
                    wrefresh(server_window);
                    closelog();
                    return(0);
                }
            exit(0);
            
         }
        /* If we are parents... */
        else if (child_pid > 0)
             close(client_socket);
        else
           {
               mvwaddstr (server_window, 2, 2, "Bytes Counter Server not running");
               mvwprintw (server_window, 3, 2, "fork(): %s", strerror(errno));
               wrefresh(server_window);
               closelog();
               return(0);
           }
    }
}

