/*
   *
   * bcount.h
   *
   * Copyright (c) 2002 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License, or
   * (at your option) any later version.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/

#include <sys/ioctl.h>
#include <netinet/ether.h>
#include <pthread.h>
#include <sys/stat.h>
#include <curses.h>
#include <fcntl.h>
#include <getopt.h>
#include <time.h>
#include <sys/time.h>


#define PROGRAM_NAME	"bcount.c"
#define VERSION		"0.6"

#ifndef PATH_SEPARATOR
#define PATH_SEPARATOR	'/'
#endif

/* If you have other MTA then replace with this one */
#define MTA		"/usr/lib/sendmail -t %s 1>/dev/null 2>/dev/null"

#define KEY_BKSP	127             /* BackSpace Key */
#define KEY_ESC		27              /* Escape Key */

#define INVISIBLE	0               /* Const for curs_set() */
#define VISIBLE		1               /* Const for curs_set() */


WINDOW *main_window;
WINDOW *void_window;
WINDOW *interface_window;
WINDOW *now_window;
WINDOW *total_window;
WINDOW *search_window;
WINDOW *mail_window;
WINDOW *mail_status_window;
WINDOW *status_window;
WINDOW *server_window;

int init_screen();
int init_screen_color();
int draw_screen();
int make_search_window();
int make_mail_window();
int close_screen();
void clrscr(void);
int fill_window (WINDOW *win);
int fill_panel (WINDOW *win);
int text_window_in_middle (WINDOW *win, char *label_window, char *place);
int get_date();
int get_file_info (char *filename);
int search_device (char *dev, int search_for); 
int refresh_now_window();
int read_iface_ip();
int get_iface_ip (char *ifname, struct sockaddr_ll *hwaddr, struct in_addr *ip);
int searching();
int total_bytes();
int summary_bytes();
int get_nr_record();
int read_log();
int check_if_old_log();
int write_log();
int check_html_file();
int check_log_read();
int check_log_append();
void rotate_dash(void);
void remove_dash(void);
void generate_html(char *filename, float tx, float rx);
int make_mail();
int sendmail (const char *from, const char *to, const char *subject, const char *text);
void* make_daemon (void* unused);



struct LOG_RECORDS
{
 char data[11];
 char interface[20];
 unsigned int rx;
 unsigned int tx;
 int record;
} log;


struct option longopts[] = {
 /* {  name   ma_arg *flag char } */
    {"interface",       1, 0, 'i'},
    {"logfile",         1, 0, 'f'},
    {"html",            1, 0, 'e'},
    {"writelog",        0, 0, 'w'},
    {"readlog",         0, 0, 'r'},
    {"summary",         0, 0, 's'},
    {"version",         0, 0, 'v'},
    {"help",            0, 0, 'h'},
    { 0, 0, 0, 0 }
};


const char *exec_name;

char *interface_ip;
char default_log_file[60] = "/var/log/bcount.log";
char *log_file = default_log_file;
char *html_file = NULL;
FILE *hf = NULL;
char current_date[11];
char get_mail_digit[10], get_mail_adress[33];
short int check_read = 0;
pthread_t thread_id;
int ip_socket;
int file_size, file_uid, file_gid;
int now_y_max, now_x_max, serv_y_max, serv_x_max;
int record_number = 0, html_answer = 0, no_more_mail = 1;



void print_version(void)
{
  printf ("BYTES COUNTER ver. \033[1;37m%s\033[0m", VERSION);
}


void print_info(void)
{
  printf ("%s: Try `%s --help or -h' for more options\n", exec_name, exec_name);
  exit(0);
}


void print_usage(void)
{
  printf ("Usage: %s [OPTIONS] ...", exec_name);
  fputs ("\nExample: bcount -i ppp0            <- Reading bytes for ppp0 in interactive mode\n\
         bcount -i ppp0 --readlog  <- Reading bytes for ppp0 from log file\n\
         If you don't choice interface then default interface will be eth0!\n\
Options:\n\
 -i [INTERFACE] --interface     Choice Interface (Default: eth0)\n\
 -f [PATH]      --logfile       Path for log file (Default: /var/log/bcount.log)\n\
 -e [PATH]    	--html		Writing Mbytes to external html file\n\
 -w      	--writelog      Write to log file\n\
 -r      	--readlog       Read from log file\n\
 -s      	--summary       Summary bytes from log file\n\
 -v, -V  	--version       Print program version\n\
 -h      	--help          Print short help\n\n\
THIS PROGRAM IS BETA VERSION. PLEASE E-MAIL ME ABOUT ANY BUG ;-)\n\n", stdout);
  exit(0);
}


void read_error(void)
{
   printf ("File \033[1;37m%s\033[0m not found!, exiting...\n", log_file);
   exit(1);
}


void write_error(void)
{
   printf ("You don't have access to write in \033[1;37m%s\033[0m, exiting...\n", log_file);
   exit(1);
}


void print_exit(void)
{
      pthread_cancel(thread_id);
      close_screen();
      clrscr();
      print_version();
      printf ("\t    Copyright (c) 2002 by Aleksander Augustowski\n");
      printf ("\t\t\t    E-mail: aleksander.augustowski@ins.com.pl\n");
      printf ("\t\t\t    Web: http://www.ins.com.pl/~alex/bcount/bcount.html\n");
      printf ("\033[1;37mExiting(0) ...\033[0m\n\n");
      exit(0);
}


/* Ctrl+C, Ctrl+\ */
static void sigterm()
{
  pthread_cancel(thread_id);
  print_exit();
  exit(0);
}


/* If little memory problem */
static void sigsegv()
{
  signal(SIGSEGV, SIG_IGN);
  pthread_cancel(thread_id);
  close_screen();
  printf ("\033[1;30m>\033[0m>\033[1;37m>\033[0m Segmentation fault \033[1;37m<\033[0m<\033[1;30m<\033[0m\n\
Please write about this to \033[1;37maleksander.augustowski@ins.com.pl\033[0m\n");
  syslog (LOG_ERR, ">>> Segmentation fault <<< Please write about this to aleksander.augustowski@ins.com.pl");
  closelog();
  exit(1);
}

