/*
   *
   * bserver.c - Bytes Counter Server
   *
   * Copyright (c) 2002 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include "common.h"
#include "bserver.h"

int search_device (char *dev, int search_for);
int make_daemon();

/* Ctrl+C, Ctrl+\ in server mode */
void sigterm_daemon()
{
    printf ("\n%d connections\nExit.\n", how_many_connections);
    closelog();
    exit(0);
}


int main()
{
  make_daemon();

  return(0);
}


/* Standalone Server for foreign connections */
int make_daemon()
{
 int alen, z, so_reuseaddr = TRUE;
 char received_version[10];
 struct sockaddr_in server_addr;
 struct sigaction sigchld_action;

/* Initialize buffer */
 char all_datas_to_send[500];

 signal(SIGTERM, sigterm_daemon);
 signal(SIGQUIT, sigterm_daemon);
 signal(SIGINT, sigterm_daemon);
 signal(SIGTSTP, sigvoid);


 printf ("Bytes Counter Server running\n\
Version %s [Linux/Intel x86]\n\
Copyright (c) 2002 aleksander.augustowski@ins.com.pl\n\
http://www.ins.com.pl/~alex/bcount/bcount.html\n\
[Press Ctrl+C to exit]\n\nWaiting for clients...\n", SVERSION);

 /* Yes ...All the Children will be dying ;-) */
 memset (&sigchld_action, 0, sizeof(sigchld_action));
 sigchld_action.sa_handler = &clean_up_child_process;
 sigaction (SIGCHLD, &sigchld_action, NULL);

 server_socket = socket (PF_INET, SOCK_STREAM, 0);
 if (server_socket < 0)
    {
       fprintf(stderr, "%s: make_daemon() -> socket()\n)", strerror(errno));
       exit(1);
    }

 /* SO_REUSEADDR help if we want run several times `bcount -d' then
    bind() function can't exit with "Address is alredy in use" :-) */
 z = setsockopt (server_socket, SOL_SOCKET, SO_REUSEADDR, &so_reuseaddr, sizeof(so_reuseaddr));
 if (z > 0)
    {
       fprintf(stderr, "%s: make_daemon() -> setsockopt()\n)", strerror(errno));
       exit(1);
    }

 /* Internet address for family addresses */
 server_addr.sin_family = PF_INET;

 /* Host order (little-endian or big-endian) 
 to network order (always big-endian) for Port */
 server_addr.sin_port = htons (SPORT);

 /* Bind socket with adress of any interface */
 server_addr.sin_addr.s_addr = INADDR_ANY;

 if (bind (server_socket, (struct sockaddr *) &server_addr, sizeof (server_addr)))
    {
       fprintf(stderr, "%s: make_daemon() -> bind()\n)", strerror(errno));
       exit(1);
    }

 if (listen (server_socket, 5))
    {
       fprintf(stderr, "%s: make_daemon() -> listen()\n)", strerror(errno));
       exit(1);
    }

 openlog ("bcount", LOG_PID, LOG_DAEMON);
 syslog (LOG_ERR, "Bytes Counter Server is running... Listening on port:%d sv=%s", SPORT, SVERSION);


 /* Neverendig story ...and Limahl :-) */
 while(1)
    {
       struct sockaddr_in client_addr;
       pid_t child_pid;
       
       alen = sizeof(server_addr);
       client_socket = accept (server_socket, (struct sockaddr *) &client_addr, &alen);
       if (client_socket < 0)
          {
             /* If get signal then continue */
             if (errno == EINTR)
                continue;
             else
                 {
            	    fprintf(stderr, "%s: make_daemon() -> accept()\n)", strerror(errno));
                    closelog();
                    exit(1);
                 }
          }

       /* Read version received from client */
       z = read (client_socket, received_version, sizeof(received_version));
       if (z < 0)
          {
             fprintf(stderr, "%s: read\n)", strerror(errno));
             exit(1);
          }

       /* On the end of received message we put NULL byte */
       received_version[z] = 0;

       if (shutdown (client_socket, SHUT_RD) == -1)
           {
              fprintf(stderr, "%s: shutdown(SHUT_RD)\n)", strerror(errno));
              exit(1);
           }

       printf ("\033[1;30m>\033[0m>\033[1;37m>\033[0m \
Connection from \033[1;37m%s\033[0m:\033[1;37m%i\033[0m\n\
     Client version \033[1;37m%s\033[0m\n", inet_ntoa (client_addr.sin_addr), ntohs (client_addr.sin_port), received_version);
       syslog (LOG_ERR, "Connection from %s:%i cv=%s", inet_ntoa (client_addr.sin_addr), ntohs (client_addr.sin_port), received_version);
       how_many_connections++;

       /* Touching data for send to the client */
       search_device (device, RX_BYTES);
       snprintf (data_to_send_rx, sizeof(data_to_send_rx), "%d", int_digit_info);
       search_device (device, TX_BYTES);
       snprintf (data_to_send_tx, sizeof(data_to_send_tx), "%d", int_digit_info);
       snprintf (all_datas_to_send, sizeof(all_datas_to_send), "%s\n%s\n%s\n%s\n", device, data_to_send_rx, data_to_send_tx, SVERSION);

       child_pid = fork();
       if (child_pid == 0)
         {
            close (STDIN_FILENO);
            close (STDOUT_FILENO);
            close (server_socket);

            z = write (client_socket, all_datas_to_send, sizeof(all_datas_to_send));
            if (z < 0)
               {
                  fprintf(stderr, "%s: write\n)", strerror(errno));
                  exit(1);
               }
               
            /* close (alen); */
            if (shutdown (client_socket, SHUT_WR) == -1)
                {
                    fprintf(stderr, "%s: shutdown(SHUT_WR)\n)", strerror(errno));
                    exit(1);
                }
            exit(0);
            
         }
        /* If we are parents... */
        else if (child_pid > 0)
             close(client_socket);
        else
           {
	       fprintf(stderr, "%s: make_daemon() -> fork()\n)", strerror(errno));
               closelog();
               exit(1);
           }
    }
}


/* Search records of device */
int search_device (char *dev, int search_for)
{

 FILE *fp;
 char buffer[1024];
 char temp_info[20];
 size_t bytes_read;
 char device_buffer[200];
 char device_test[20];
 char* temp_buffer;
 int device_buffer_lenght = 0, records = 0;
 int flag = 666, n = 0, i = 0, c = 0;

 if ((fp = fopen(DEV_FILE, "r")) == NULL)
    {
        printf ("File %s not found.\n\
This program don't execute without \"proc file system\", exiting...\n", DEV_FILE);
        perror ("search_device()");
    }

  fseek (fp, 199L, SEEK_SET); /* 199 Bytes have trash texts in log_file */
  bytes_read = fread (buffer, 1, sizeof(buffer), fp);
  fclose(fp);

  if (bytes_read == 0 || bytes_read == sizeof(buffer))
    {
      perror ("search_device(): Buffer underrun");
      exit(1);
    }

  buffer[bytes_read] = '\0';
  temp_buffer = strstr (buffer, dev);
  if (temp_buffer == NULL)
     device_error();
  
  while (temp_buffer[n] != ':') 
    n++;
    
  strncpy(device_test, temp_buffer, n);
    
  device_test[n] = '\0';
  
  if (strcmp(device_test, dev))
     device_error();

  while (search_for == flag)    /* Decide: exit or not */
     return(0);

  while (temp_buffer[n] != '\n') 
    n++;

  strncpy(device_buffer, temp_buffer, n);

  n = 0;
  

  while (device_buffer[n] != ':')
     n++;

  device_buffer_lenght = strlen(device_buffer);
  while (n != device_buffer_lenght)
     {
        if ((isspace(device_buffer[n])) && (isdigit(device_buffer[n+1])) == 0)
             ;
             else
                 {
                    if (((isspace(device_buffer[n])) && (isdigit(device_buffer[n+1])) != 0) || ((device_buffer[n] == ':') && (isdigit(device_buffer[n+1]) != 0)))
                      {
                          records++;
                          if (records == search_for)
                             {
                                i = n+1;
                                while ((isspace(device_buffer[i])) == 0)
                                    {
                                        temp_info[c] = device_buffer[i];
                                        c++;
                                        i++;
                                    }
                             }

                      }

                 }
        n++;
      }


  temp_info[c] = '\0';

   
  int_digit_info = atoi(temp_info);
   float_digit_bytes = atof(temp_info);
    float_digit_Kbytes = float_digit_bytes / 1024;
     float_digit_Mbytes = float_digit_Kbytes / 1024;
   
 return(0);
}

